/*The source is rather a mess so here is some info about
/title/00000001/00000002/data/cdb.vff, which is where messageboard data
is stored.  This may be inacurrate, use with caution.

0x00000	string[4] 56 46 46 20 ("VFF ")
0x00004	u16       FE FF (Byte-order mark)
0x00006	0x1A bytes of unknown data padded with zeroes
0x00020 0x14000 bytes of unknown data padded with zeroes
0x14020	0x14000 bytes of unknown data padded with zeroes
0x28020 0x1000 bytes of data padded with zeroes.  In this data are strings
		containing years, e.g "2009".  Only years for which messages are on the
		board appear.

None of that is particularly interesting for the purposes of this program.
Then we get:

0x29020 u32 last_message_id.  Each message posted has it's own message_id
		starting from zero.  Here is the id of the most recent message.
0x29024 0xFFC bytes of unknown data.
0x2a020 string[8] 43 44 42 46 49 4C 45 02 ("CDBFILE ")

Throughout the rest of the file there are records starting with CDBFILE.  A
CDBFILE is an individual message on the Wii message board.  Here is some info
on it's layout.  They always start on a 0x100 + 0x20 boundary.

0x000	string[8] 	43 44 42 46 49 4c 45 02 ("CDBFILE ")

0x008	0x08 bytes	unknown

0x010   u32			message_type - Seems to be 12 00 00 00 for emails and
					0C 00 00 00 for playtimelog.

0x014	string[92]  "playtimelog" for Today's Accomplishments message or
					"ripl_board_record" for emails and messages from games.

0x070	u32			message_id

0x074	u32			If playtimelog, this is the number of times a channel or
					game was loaded.  If this number is higher than ten, only
					ten entries will be listed.  For other messages, purpose is
					unknown.

0x078	z32			zero

0x07B	u32			Time message was last modified, in number of seconds since
					00:00 01/01/2000

0x080   z[896]		0x380 bytes of zeroes.

0x400	string[4]	52 49 5F 35 ("RI_5")

0x404	0x0C bytes	unknown.  zeroes in playtimelog.

0x410	u64			Time message was last modified, in "ticks" since 00:00
					01/01/2000.  There are 60,750,000 "ticks" per second.

0x418	0x08 bytes	unknown.  zeroes in playtimelog

0x420   z[240]		0xF0 bytes of zeroes.

0x510	0x0C bytes  unknown.

0x51C	u32			Length of data between "RI_5" string and start of message
					text, this is always 0x148.

0x520	u32			unknown but is always 0x178 in playtimelog.

0x524	0x08 bytes	unknown

0x52C	u32			In playtimelog only, length of data between "RI_5" string
					and end of message text (which is padded with zeroes at the
					end)

0x530	u32			unknown, always 0x668 in playtimelog.

0x534	0x14 bytes  unknown

0x548	wstring		Unicode string containing text of message, padded with
					zeroes to length specified at 0x52C.

THE REST OF THIS IS ONLY APPLICABLE TO PLAYTIMELOG:

After the string there is a four byte code.  If this code is 30 33 5F 30
("03_0"), it is followed by 12 bytes of unknown data; if the code is
00 00 00 01, it is followed by 4 bytes of unknown data.  Following this is a
record for each game or channel in the log, to a maximum of ten records.

struct logrecord
{
	u32 unknown;
	wchar title[21]; (twenty 16-bit characters and null terminator.  this is
					what is shown when you hover over the channel)
	wchar extended_title[21]; (sometimes there is an extension to the title
								shown as a new line on the message board)
	u64 start_time; (see above at 0x410 for this format)
	u64 end_time;
	char title_id[4]; (4 character code for the title)
	char code[2]; (2 character code of some sort, not sure what this is for)
	char unknown[18]; (always zeroes?)
	u64 others; this is zero unless the value above at 0x74 is greater than 10,
				in which case this is the time listed as "others" on the message.
}*/

#define GFX  //for console mode debugging
//#define SSHOT //Screenshot code
//#define SD //read from sd:/cdb.vff instead
#define BUILD_DATE "19-05-2009"
#define BUILD_VER "v1.02"
#define MAXTITLES 2048

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <ogcsys.h>
#include <gccore.h>
#include <wiiuse/wpad.h>
#include <fat.h>
#include <ogc/isfs.h>

#ifdef GFX
#include "GRRLIB.h"
#include "resource.h"
#include "andale.h"
#endif

struct {
	unsigned int unknown;
	char title[42];
	char ext_title[42];
	unsigned long long int start_time;
	unsigned long long int end_time;
	char title_id[4];
	char code[2];
	char unknown2[18];
	unsigned long long int others;
} logrecord;


typedef struct element element;
struct element {
	char title[80];
	unsigned long long int playtime;
	unsigned long long int firstplay;
	unsigned long long int lastplay;
	unsigned long long int averageplay;
	int timesplayed;
};
element *playlist[MAXTITLES],total;

enum {LAUNCHER,WIIMENU,POWEROFF};

#ifdef GFX
Mtx GXmodelView2D;
ir_t WiiCursor;
unsigned char *andale, *bg, *title, *radio, *chk, *cursor;
#endif
unsigned int wpaddown;
long long int currentplay, other=0;
int where;
short numtitles=0;
char power=0,cursor_on;
#ifndef SD
char readbuffer[1024] ATTRIBUTE_ALIGN(32);  //doesn't work if this is not global, probaly something wrong elsewhere.
unsigned int file_pos=0;
#endif

unsigned int sleep(unsigned int seconds);

void quit(char mode)
{
#ifdef GFX
	GRRLIB_Stop();
#endif
	WPAD_Shutdown();
	if (mode==LAUNCHER)
		exit(0);
	else if(mode==WIIMENU)
		SYS_ResetSystem(SYS_RETURNTOMENU,0,0);
	else
	{
		SYS_ResetSystem(SYS_POWEROFF,0,0);
		exit(0);
	}
}

int wlen(short* str)
{
	short *ptr;
	ptr=str;
	while(*ptr)
	{
		ptr++;
		if((ptr-str)>1024)
			return -1;
	}
	return ptr-str;
}

void narrow(char* narr, short* wide)
{
	int i;
	char *nptr;
	short *wptr;
	nptr=narr;
	wptr=wide;
	for(i=0;i<wlen(wide);i++)
		*nptr++=(char)*wptr++;
	*nptr=0;
}

void read(void *ptr,int len,int size,void *file)
{
#ifndef SD
	int *fptr;
	fptr=file;
	ISFS_Read(*fptr,readbuffer,len*size);
	memcpy(ptr,readbuffer,len*size);
	file_pos+=len*size;
#else
	fread(ptr,len,size,(FILE*)file);
#endif
}

void seek(void *file, int pos, int mode)
{
#ifndef SD
	int *fptr;
	fptr=file;
	ISFS_Seek(*fptr,pos,mode);
	if(mode==SEEK_SET)
		file_pos=pos;
	else if(mode==SEEK_CUR)
		file_pos+=pos;
#else
	fseek((FILE*)file,pos,mode);
#endif
}

int cmp_timesplayed(const void *a, const void *b)
{
	element **p,**q,*r,*s;
	p=(element**)a;
	q=(element**)b;
	r=*p;
	s=*q;
	return s->timesplayed-r->timesplayed;
}

int cmp_playtime(const void *a, const void *b)
{
	element **p,**q,*r,*s;
	long long int t;
	p=(element**)a;
	q=(element**)b;
	r=*p;
	s=*q;
	t=s->playtime-r->playtime;
	if(t<0)
		return -1;
	else if(t==0)
		return 0;
	else
		return 1;
}

int cmp_averageplay(const void *a, const void *b)
{
	element **p,**q,*r,*s;
	long long int t;
	p=(element**)a;
	q=(element**)b;
	r=*p;
	s=*q;
	t=s->averageplay-r->averageplay;
	if(t<0)
		return -1;
	else if(t==0)
		return 0;
	else
		return 1;
}

int cmp_lastplay(const void *a, const void *b)
{
	element **p,**q,*r,*s;
	long long int t;
	p=(element**)a;
	q=(element**)b;
	r=*p;
	s=*q;
	t=s->lastplay-r->lastplay;
	if(t<0)
		return -1;
	else if(t==0)
		return 0;
	else
		return 1;
}

int cmp_firstplay(const void *a, const void *b)
{
	element **p,**q,*r,*s;
	long long int t;
	p=(element**)a;
	q=(element**)b;
	r=*p;
	s=*q;
	t=s->firstplay-r->firstplay;
	if(t<0)
		return -1;
	else if(t==0)
		return 0;
	else
		return 1;
}

int cmpd_timesplayed(const void *a, const void *b)
{
	element **p,**q,*r,*s;
	p=(element**)a;
	q=(element**)b;
	r=*p;
	s=*q;
	return r->timesplayed-s->timesplayed;
}

int cmpd_playtime(const void *a, const void *b)
{
	element **p,**q,*r,*s;
	long long int t;
	p=(element**)a;
	q=(element**)b;
	r=*p;
	s=*q;
	t=r->playtime-s->playtime;
	if(t<0)
		return -1;
	else if(t==0)
		return 0;
	else
		return 1;
}

int cmpd_averageplay(const void *a, const void *b)
{
	element **p,**q,*r,*s;
	long long int t;
	p=(element**)a;
	q=(element**)b;
	r=*p;
	s=*q;
	t=r->averageplay-s->averageplay;
	if(t<0)
		return -1;
	else if(t==0)
		return 0;
	else
		return 1;
}

int cmpd_lastplay(const void *a, const void *b)
{
	element **p,**q,*r,*s;
	long long int t;
	p=(element**)a;
	q=(element**)b;
	r=*p;
	s=*q;
	t=r->lastplay-s->lastplay;
	if(t<0)
		return -1;
	else if(t==0)
		return 0;
	else
		return 1;
}

int cmpd_firstplay(const void *a, const void *b)
{
	element **p,**q,*r,*s;
	long long int t;
	p=(element**)a;
	q=(element**)b;
	r=*p;
	s=*q;
	t=r->firstplay-s->firstplay;
	if(t<0)
		return -1;
	else if(t==0)
		return 0;
	else
		return 1;
}

void time_format(char *s,long long int t)
{
	unsigned int days, hours, mins, secs;
	days=(unsigned int)(t/60750000);
	secs=days%60;
	days-=secs;
	days/=60;
	mins=days%60;
	days-=mins;
	days/=60;
	hours=days%24;
	days-=hours;
	days/=24;
	if(days)
		sprintf(s,"%udys %02uhrs %02umins %02usecs",days,hours,mins,secs);
	else if(hours)
		sprintf(s,"%02uhrs %02umins %02usecs",hours,mins,secs);
	else
		sprintf(s,"%02umins %02usecs",mins,secs);
}

void date_format(char *s,long long int t)
{
	short dmonth[13];
	unsigned int i, n, qyear, qday,lyear=0, year, month=0, day=0, hour, minute, second;
	dmonth[0]=0;
	dmonth[1]=31;
	dmonth[2]=59;
	dmonth[3]=90;
	dmonth[4]=120;
	dmonth[5]=151;
	dmonth[6]=181;
	dmonth[7]=212;
	dmonth[8]=243;
	dmonth[9]=273;
	dmonth[10]=304;
	dmonth[11]=334;
	dmonth[12]=365;
	n=(unsigned int)(t/60750000);
	second=n%60;
	n-=second;
	n/=60;
	minute=n%60;
	n-=minute;
	n/=60;
	hour=n%24;
	n-=hour;
	n/=24;
	n+=1;
	qday=n%1461;
	qyear=(n-qday)/1461;
	if(qday>=60)
		qyear+=1;
	year=(n-qyear)/365;
	n-=qyear*366;
	n-=(year-qyear)*365;
	if(year%4==0)
		lyear=1;
	year+=2000;
	for(i=0;i<=12;i++)
		if(n<=(dmonth[i]+(i>1?lyear:0)))
		{
			month=i;
			day=n-(dmonth[i-1]+(i>2?lyear:0));
			break;
		}
	sprintf(s,"%02u/%02u/%4u %02u:%02u:%02u",day,month,year,hour,minute,second);
}

void init_console()
{
	static void *xfb = NULL;
	static GXRModeObj *rmode = NULL;
	rmode = VIDEO_GetPreferredMode(NULL);
	xfb = MEM_K0_TO_K1(SYS_AllocateFramebuffer(rmode));
	console_init(xfb,20,20,rmode->fbWidth,rmode->xfbHeight,rmode->fbWidth*VI_DISPLAY_PIX_SZ);
	VIDEO_Configure(rmode);
	VIDEO_SetNextFramebuffer(xfb);
	VIDEO_SetBlack(FALSE);
	VIDEO_Flush();
	VIDEO_WaitVSync();
	if(rmode->viTVMode&VI_NON_INTERLACE) VIDEO_WaitVSync();
}

void error(char *s)
{
	init_console();
	printf("\nPlaystats %s    www.toxicbreakfast.com  TOX010  %s\n\n",BUILD_VER,BUILD_DATE);
	printf("%s\n\nExiting...",s);
	sleep(5);
	exit(1);
}

int percent(int a,int b)
{
	float f,g;
	f=(float)a;
	g=(float)b;
	return (int)((f/g)*100);
}

#ifdef GFX
void splash()
{
	while(1)
	{
		WPAD_ScanPads();
		wpaddown = WPAD_ButtonsDown(0);
		GRRLIB_DrawImg(160,120,320,240,bg,0,2,2,255);
		GRRLIB_DrawImg(170,190,396,56,title,0,1,1,255);
		GRRLIB_Printf(400,15,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,"%s %s",BUILD_VER,BUILD_DATE);
		GRRLIB_Printf(160,400,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,1,"Press A to start");
		GRRLIB_Render();
		if (wpaddown & WPAD_BUTTON_HOME) quit(LAUNCHER);
		else if(wpaddown & WPAD_BUTTON_A) return;
#ifdef SSHOT
		if(wpaddown&WPAD_BUTTON_2)
			GRRLIB_ScrShot("sd:/screen.png");
#endif
		if(power==1)
			quit(POWEROFF);
		else if(power==2)
			quit(WIIMENU);
	}
}

void showlist(char sortby, char order)
{
	char header[80], buf[80], format[80], highlighted=0;
	int i, offset=0;
	if(order==0)
		switch(sortby)
		{
			case 1:
				qsort(playlist,numtitles,sizeof(element*),cmp_timesplayed);
				strcpy(header,"MOST TIMES PLAYED");
				break;
			case 2:
				qsort(playlist,numtitles,sizeof(element*),cmp_playtime);
				strcpy(header,"LONGEST TIME PLAYED");
				break;
			case 3:
				qsort(playlist,numtitles,sizeof(element*),cmp_averageplay);
				strcpy(header,"LONGEST AVERAGE PLAY");
				break;
			case 4:
				qsort(playlist,numtitles,sizeof(element*),cmp_lastplay);
				strcpy(header,"MOST RECENTLY PLAYED");
				break;
			case 5:
				qsort(playlist,numtitles,sizeof(element*),cmp_firstplay);
				strcpy(header,"NEWEST GAME");
				break;
		}
	else
		switch(sortby)
		{
			case 1:
				qsort(playlist,numtitles,sizeof(element*),cmpd_timesplayed);
				strcpy(header,"LEAST TIMES PLAYED");
				break;
			case 2:
				qsort(playlist,numtitles,sizeof(element*),cmpd_playtime);
				strcpy(header,"SHORTEST TIME PLAYED");
				break;
			case 3:
				qsort(playlist,numtitles,sizeof(element*),cmpd_averageplay);
				strcpy(header,"SHORTEST AVERAGE PLAY");
				break;
			case 4:
				qsort(playlist,numtitles,sizeof(element*),cmpd_lastplay);
				strcpy(header,"LEAST RECENTLY PLAYED");
				break;
			case 5:
				qsort(playlist,numtitles,sizeof(element*),cmpd_firstplay);
				strcpy(header,"OLDEST GAME");
				break;
		}
	while(1)
	{
		WPAD_ScanPads();
		wpaddown = WPAD_ButtonsDown(0);
		WPAD_IR(0,&WiiCursor);
        if(WiiCursor.valid==0) cursor_on=0;
        else cursor_on=1;
        if(WiiCursor.y>=375&&WiiCursor.y<415)
        {
        	if(WiiCursor.x>=275&&WiiCursor.x<365)
        		highlighted=1;
        	else if(WiiCursor.x>=100&&WiiCursor.x<190&&offset>0)
        		highlighted=2;
        	else if(WiiCursor.x>=450&&WiiCursor.x<540&&offset+12<numtitles)
        		highlighted=3;
        	else
        		highlighted=0;
		}
		else highlighted=0;
        if(wpaddown&WPAD_BUTTON_A&&highlighted==1)
        	return;
        else if(offset>0&&((wpaddown&WPAD_BUTTON_A&&highlighted==2)||wpaddown&WPAD_BUTTON_MINUS))
        	offset-=12;
        else if(offset+12<numtitles&&((wpaddown&WPAD_BUTTON_A&&highlighted==3)||wpaddown&WPAD_BUTTON_PLUS))
        	offset+=12;
#ifdef SSHOT
		if(wpaddown&WPAD_BUTTON_2)
			GRRLIB_ScrShot("sd:/screen.png");
#endif
		GRRLIB_FillScreen(0xFFD0E0FF);
		GRRLIB_DrawImg(160,120,320,240,bg,0,2,2,96);
		GRRLIB_DrawImg(170,190,396,56,title,0,1,1,96);
		GRRLIB_Printf(320-strlen(header)*5,50,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,header);
		for(i=offset;i<offset+12&&i<numtitles;i++)
		{
			narrow(buf,(short*)playlist[i]->title);
			if(sortby==1)
				GRRLIB_Printf(40,100+(i-offset)*20,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,"%03d: %20s %5u times  %6.2f%%",i+1,buf,playlist[i]->timesplayed,((float)playlist[i]->timesplayed/total.timesplayed)*100);
			else if(sortby==2)
			{
				time_format(format,playlist[i]->playtime);
				GRRLIB_Printf(10,100+(i-offset)*20,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,"%03d: %20s %26s %6.2f%%",i+1,buf,format,((long double)playlist[i]->playtime/(total.playtime+other))*100);
			}
			else if(sortby==3)
			{
				time_format(format,playlist[i]->averageplay);
				GRRLIB_Printf(40,100+(i-offset)*20,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,"%03d: %20s %26s",i+1,buf,format);
			}
			else if(sortby==4)
			{
				date_format(format,playlist[i]->lastplay);
				GRRLIB_Printf(40,100+(i-offset)*20,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,"%03d: %20s   %s",i+1,buf,format);
			}
			else if(sortby==5)
			{
				date_format(format,playlist[i]->firstplay);
				GRRLIB_Printf(40,100+(i-offset)*20,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,"%03d: %20s   %s",i+1,buf,format);
			}
		}
		if(offset>0)
		{
			GRRLIB_Rectangle(100,375,90,40,0xFF000000,0);
			GRRLIB_Rectangle(101,376,88,38,0xFF000000,0);
			if(highlighted==2)
				GRRLIB_Rectangle(102,377,86,36,0xFFFFFF00,1);
			GRRLIB_Printf(107,377,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,1,"Prev");
		}
		GRRLIB_Rectangle(275,375,90,40,0xFF000000,0);
		GRRLIB_Rectangle(276,376,88,38,0xFF000000,0);
		if(highlighted==1)
			GRRLIB_Rectangle(277,377,86,36,0xFFFFFF00,1);
		GRRLIB_Printf(282,377,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,1,"Back");
		if(offset+12<numtitles)
		{
			GRRLIB_Rectangle(450,375,90,40,0xFF000000,0);
			GRRLIB_Rectangle(451,376,88,38,0xFF000000,0);
			if(highlighted==3)
				GRRLIB_Rectangle(452,377,86,36,0xFFFFFF00,1);
			GRRLIB_Printf(457,377,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,1,"Next");
		}
		if(cursor_on) GRRLIB_DrawImg(WiiCursor.x,WiiCursor.y,32,64,cursor,0,1,1,255);
		GRRLIB_Render();
		if (wpaddown & WPAD_BUTTON_HOME) quit(LAUNCHER);
		if(power==1)
			quit(POWEROFF);
		else if(power==2)
			quit(WIIMENU);
	}
}

void showtotal()
{
	char atm[40], ttm[40], fdt[40], ldt[40], highlighted=0;
	time_format(atm,total.averageplay);
	time_format(ttm,total.playtime+other);
	date_format(fdt,total.firstplay);
	date_format(ldt,total.lastplay);
	while(1)
	{
		WPAD_ScanPads();
		wpaddown = WPAD_ButtonsDown(0);
		WPAD_IR(0,&WiiCursor);
        if(WiiCursor.valid==0) cursor_on=0;
        else cursor_on=1;
        if(WiiCursor.x>=275&&WiiCursor.x<365&&WiiCursor.y>=375&&WiiCursor.y<415)
        	highlighted=1;
        else
        	highlighted=0;
        if(wpaddown&WPAD_BUTTON_A&&highlighted)
        	return;
		GRRLIB_FillScreen(0xFFD0E0FF);
		GRRLIB_DrawImg(160,120,320,240,bg,0,2,2,96);
		GRRLIB_DrawImg(170,190,396,56,title,0,1,1,96);
		GRRLIB_Printf(100,100,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,"Total sessions: %u",total.timesplayed);
		GRRLIB_Printf(100,140,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,"Total time played: %s",ttm);
		GRRLIB_Printf(100,180,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,"Average time per session: %s",atm);
		GRRLIB_Printf(100,220,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,"First play: %s",fdt);
		GRRLIB_Printf(100,260,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,"Last play: %s",ldt);
		GRRLIB_Printf(100,300,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,"Number of titles played: %u",numtitles);
		GRRLIB_Rectangle(275,375,90,40,0xFF000000,0);
		GRRLIB_Rectangle(276,376,88,38,0xFF000000,0);
		if(highlighted==1)
			GRRLIB_Rectangle(277,377,86,36,0xFFFFFF00,1);
		GRRLIB_Printf(282,377,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,1,"Back");
		if(cursor_on) GRRLIB_DrawImg(WiiCursor.x,WiiCursor.y,32,64,cursor,0,1,1,255);
		GRRLIB_Render();
		if (wpaddown & WPAD_BUTTON_HOME) quit(LAUNCHER);
		if(power==1)
			quit(POWEROFF);
		else if(power==2)
			quit(WIIMENU);
	}
}
#endif

void powerbutton1(){
	power=1;}
void powerbutton2(int channel){
	power=1;}
void resetbutton(){
	power=2;}

int main(){
	int i,j, pcnt=0,region;
#ifdef SD
	FILE *fp;
#else
	int *fp,ifp;
#endif
	char sortby=1,order=0,highlighted=0;
	unsigned int total_messages, message_id, play_entries, message_end, logrecord_header;
	char buf[80];
    VIDEO_Init();
    //Don't support Japanese/Korean characters yet.
    region=CONF_GetRegion();
    if(region==0)
    	error("Sorry, this application is not supported in Japan.");
    else if(region==3) //Korea?  Not sure.
    	error("Sorry, this application is not supported in Korea.");
	memset(playlist,0,sizeof(playlist));
	memset(&total,0,sizeof(element));
	strcpy(total.title,"_TOTAL");
#ifdef GFX
    GRRLIB_InitVideo();
	andale=GRRLIB_LoadTexture(font_andale);
	bg=GRRLIB_LoadTexture(img_title1);
	title=GRRLIB_LoadTexture(img_title2);
	radio=GRRLIB_LoadTexture(img_radio);
	chk=GRRLIB_LoadTexture(img_radiochk);
	cursor=GRRLIB_LoadTexture(img_cursor);
#endif
    WPAD_Init();
#ifdef GFX
	WPAD_SetVRes(0, 640, 480);
	WPAD_SetDataFormat(0, WPAD_FMT_BTNS_ACC_IR);
#endif
	SYS_SetResetCallback(resetbutton);
	SYS_SetPowerCallback(powerbutton1);
	WPAD_SetPowerButtonCallback(powerbutton2);
#ifndef GFX
	init_console();
	printf("\n");
#endif
#ifdef SD
	fatInitDefault();
	if((fp=fopen("sd:/cdb.vff","rb")))
	{
#else
	ISFS_Initialize();
	if((ifp=ISFS_Open("/title/00000001/00000002/data/cdb.vff",ISFS_OPEN_READ)))
	{
		fp=&ifp;
#endif
		seek(fp,0x29020,SEEK_SET);
		read(&total_messages,4,1,fp);
		seek(fp,0x1004,SEEK_CUR);
		while(1)
		{
			seek(fp,0xC,SEEK_CUR);
			read(buf,1,12,fp);
			buf[11]=0;
			seek(fp,0x50,SEEK_CUR);
			read(&message_id,4,1,fp);
			i=percent(message_id,total_messages);
			if(i>pcnt)
				pcnt=i;
#ifdef GFX
			GRRLIB_DrawImg(160,120,320,240,bg,0,2,2,255);
			GRRLIB_DrawImg(170,190,396,56,title,0,1,1,255);
			GRRLIB_Printf(400,15,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,"%s %s",BUILD_VER,BUILD_DATE);
			//loading status not accurate as message_ids not necessarily sequential but at least it looks like it is doing something.
			GRRLIB_Printf(180,400,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,1,"Loading...%d%%",pcnt);
			GRRLIB_Render();
#endif
			if(strcmp(buf,"playtimelog")==0)
			{
				read(&play_entries,4,1,fp);
				if(play_entries>10)
					play_entries=10;
				seek(fp,0x4B4,SEEK_CUR);
				read(&message_end,4,1,fp);
				message_end-=0x130;
				seek(fp,message_end,SEEK_CUR);
				read(&logrecord_header,4,1,fp);
				if(logrecord_header==1)
					seek(fp,0x04,SEEK_CUR);
				else
					seek(fp,0x0C,SEEK_CUR);
				for(i=0;i<play_entries;i++)
				{
					read(&logrecord,sizeof(logrecord),1,fp);
					for(j=0;;j++)
					{
						if(j==MAXTITLES)
							error("Error: MAXTITLES reached.");
						if(playlist[j]==0)
						{
							numtitles++;
							playlist[j]=(element*)malloc(sizeof(element));
							memcpy(playlist[j]->title,logrecord.title,80);
							playlist[j]->playtime=0;
							playlist[j]->firstplay=logrecord.start_time;
							if(total.firstplay==0)
								total.firstplay=logrecord.start_time;
							playlist[j]->timesplayed=0;
							break;
						}
						else if(memcmp(logrecord.title,playlist[j]->title,wlen((short *)logrecord.title)*2)==0)
							break;
					}
					currentplay=logrecord.end_time-logrecord.start_time;
					playlist[j]->playtime+=currentplay;
					total.lastplay=logrecord.end_time;
					playlist[j]->timesplayed++;
					total.playtime+=currentplay;
					if((int)logrecord.others>0)
						other+=logrecord.others;
					playlist[j]->lastplay=logrecord.end_time;
					total.timesplayed++;
				}
			}
			if(message_id==total_messages)
				break;
#ifdef SD
			where=(((int)((ftell(fp)-0x21)/0x100))+1)*0x100+0x20;
#else
			where=(((int)((file_pos-0x21)/0x100))+1)*0x100+0x20;
#endif
			while(1)
			{
				seek(fp,where,SEEK_SET);
				read(&buf,8,1,fp);
				buf[7]=0;
				if(strcmp(buf,"CDBFILE")==0)
					break;
				else
					where+=0x100;
			}
		}
#ifdef SD
		fclose(fp);
#else
		ISFS_Close(ifp);
		ISFS_Deinitialize();
#endif
	}
	else
		error("Error reading message board");
	total.averageplay=(unsigned long long int)total.playtime/total.timesplayed;
	for(i=0;i<numtitles;i++)
		playlist[i]->averageplay=(unsigned long long int)playlist[i]->playtime/playlist[i]->timesplayed;

#ifdef GFX
	splash();
#endif
	while(1)
	{
		WPAD_ScanPads();
		wpaddown = WPAD_ButtonsDown(0);
#ifdef GFX
		WPAD_IR(0,&WiiCursor);
        if(WiiCursor.valid==0) cursor_on=0;
        else cursor_on=1;
        if(WiiCursor.x>=375&&WiiCursor.x<444&&WiiCursor.y>=312&&WiiCursor.y<352)
        	highlighted=1;
        else if(WiiCursor.x>=375&&WiiCursor.x<585&&WiiCursor.y>=372&&WiiCursor.y<412)
        	highlighted=2;
        else
        	highlighted=0;
        if((wpaddown&WPAD_BUTTON_A)&&cursor_on)
        {
			if(WiiCursor.x>=20&&WiiCursor.x<52&&WiiCursor.y>=120&&WiiCursor.y<312&&((int)WiiCursor.y%40)<32)
				sortby=(char)((WiiCursor.y-120)/40)+1;
			else if(WiiCursor.x>=360&&WiiCursor.x<392&&WiiCursor.y>=120&&WiiCursor.y<192&&((int)WiiCursor.y%40)<32)
				order=(char)((WiiCursor.y-120)/40);
			else if(highlighted==1)
				showlist(sortby,order);
			else if(highlighted==2)
				showtotal();
		}
#ifdef SSHOT
		if(wpaddown&WPAD_BUTTON_2)
			GRRLIB_ScrShot("sd:/screen.png");
#endif
		GRRLIB_FillScreen(0xFFD0E0FF);
		GRRLIB_DrawImg(160,120,320,240,bg,0,2,2,96);
		GRRLIB_DrawImg(170,190,396,56,title,0,1,1,96);
		for(i=1;i<6;i++)
			GRRLIB_DrawImg(20,i*40+80,32,32,sortby==i?chk:radio,0,1,1,255);
		for(i=0;i<2;i++)
			GRRLIB_DrawImg(360,i*40+120,32,32,order==i?chk:radio,0,1,1,255);
		GRRLIB_Printf(20,50,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,1,"Sort by:");
		GRRLIB_Printf(60,120,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,"Number of times played");
		GRRLIB_Printf(60,160,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,"Total time played");
		GRRLIB_Printf(60,200,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,"Average length per play");
		GRRLIB_Printf(60,240,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,"Last time played");
		GRRLIB_Printf(60,280,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,"First time played");
		GRRLIB_Printf(360,50,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,1,"Order:");
		GRRLIB_Printf(400,120,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,"Ascending");
		GRRLIB_Printf(400,160,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,0.5,"Descending");
		GRRLIB_Rectangle(375,312,69,40,0xFF000000,0);
		GRRLIB_Rectangle(376,313,67,38,0xFF000000,0);
		if(highlighted==1)
			GRRLIB_Rectangle(377,314,65,36,0xFFFFFF00,1);
		GRRLIB_Printf(382,314,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,1,"GO!");
		GRRLIB_Rectangle(375,372,210,40,0xFF000000,0);
		GRRLIB_Rectangle(376,373,208,38,0xFF000000,0);
		if(highlighted==2)
			GRRLIB_Rectangle(377,374,206,36,0xFFFFFF00,1);
		GRRLIB_Printf(382,374,andale,W_ANDALE,H_ANDALE,(char*)fontmap_andale,WC_ANDALE,HC_ANDALE,0xFF000000,1,"Show total");
		if(cursor_on) GRRLIB_DrawImg(WiiCursor.x,WiiCursor.y,32,64,cursor,0,1,1,255);
		GRRLIB_Render();
#endif
		if (wpaddown & WPAD_BUTTON_HOME) quit(LAUNCHER);
		if(power==1)
			quit(POWEROFF);
		else if(power==2)
			quit(WIIMENU);
    }
    return 0;
}
